@@ -1,10 +1,11 @@
 """Functions to parse data in the rather bespoke format used by the MCD."""
+import re
+from collections import namedtuple
+from datetime import datetime
 from pathlib import Path
 from typing import List, Union
-from datetime import datetime
-from collections import namedtuple
+
 import numpy as np
-import re
 
 
 def parse_number(num: str) -> Union[float, int, None]:
@@ -40,37 +41,31 @@ def parse_header(lines: List[str]) -> dict:
     # written to be readable by people beginning python, so rather verbose.
     data = {}
     match = re.search("MCD_(.+) with (.+).", lines[0])
-    if match:
-        data["mcd_version"] = match.group(1)
-        data["model"] = match.group(2)
+    data["mcd_version"] = match.group(1)  # type: ignore
+    data["model"] = match.group(2)  # type: ignore
     match = re.search("Ls (.+). Altitude (.+) ALS Local time (.+)", lines[1])
-    if match:
-        data["ls"] = match.group(1)
-        data["altitude"] = match.group(2)
-        data["local_time"] = match.group(3).strip()
+    data["ls"] = match.group(1)  # type: ignore
+    data["altitude"] = match.group(2)  # type: ignore
+    data["local_time"] = match.group(3).strip()  # type: ignore
     assert "-" * 6 in lines[2]
     match = re.search("Column 1 is (.+)", lines[3])
-    if match:
-        data["column_1"] = match.group(1)
+    data["column_1"] = match.group(1)  # type: ignore
 
     match = re.search(r"Columns 2\+ are (.+)", lines[4])
-    if match:
-        data["variable"] = match.group(1)
+    data["variable"] = match.group(1)  # type: ignore
 
     match = re.search("Line 1 is (.+)", lines[5])
-    if match:
-        data["keys"] = match.group(1)
+    data["keys"] = match.group(1)  # type: ignore
     assert "-" * 6 in lines[6]
     match = re.search("Retrieved on: (.+)", lines[7])
-    if match:
-        data["retrieval_date"] = datetime.fromisoformat(match.group(1))
+    data["retrieval_date"] = datetime.fromisoformat(match.group(1))  # type: ignore
     return data
 
 
-DataTable = namedtuple("DataTable", ["data", "xlabels", "ylabels"])
+_DataTable = namedtuple("_DataTable", ["data", "xlabels", "ylabels"])
 
 
-def parse_body(body: List[str]) -> "DataTable":
+def parse_body(body: List[str]) -> "_DataTable":
     """
     Parse body of data from the MCD.
 
@@ -78,7 +73,7 @@ def parse_body(body: List[str]) -> "DataTable":
         body (List[str]): lines to parse.
 
     Returns:
-        (DataTable): The parsed data.
+        (_DataTable): The parsed data.
     """
     # here we use the map (/reduce, but here we don't reduce) paradigm
     # to show how sometimes functional programming is a *lot* simpler
@@ -90,16 +85,15 @@ def parse_body(body: List[str]) -> "DataTable":
     # numpy has it's own map/reduce fns which are implemented in C
     # and can be a lot faster than python's.
 
-    body = map(lambda row: " ".join(row.strip().split()), body)
-    body = list(body)
+    body = list(map(lambda row: " ".join(row.strip().split()), body))
     xlabels = body[0].split("||")[1].strip().split(" ")
     body = body[2:]
-    xlabels = map(parse_number, xlabels)
-    ylabels = map(lambda row: row.split("||")[0].strip(), body)
-    ylabels = map(parse_number, ylabels)
-    data = map(lambda row: row.split("||")[1].strip().split(" "), body)
-    data = np.array(list(data))
-    return DataTable(data, list(xlabels), list(ylabels))
+    xlabels = map(parse_number, xlabels)  # type: ignore
+    ylabels_map = map(lambda row: row.split("||")[0].strip(), body)
+    ylabels = map(parse_number, ylabels_map)
+    data_map = map(lambda row: row.split("||")[1].strip().split(" "), body)
+    data = np.array(list(data_map), dtype=float)
+    return _DataTable(np.rot90(data), list(xlabels), list(ylabels))
 
 
 def read_ascii_data(dataf: Path) -> dict:
@@ -112,6 +106,9 @@ def read_ascii_data(dataf: Path) -> dict:
     Returns:
         (dict): The data.
 
+    Raises:
+        ValueError: Failed to parse supplied file.
+
     """
     sections = {}
     with dataf.open() as f:
@@ -119,22 +116,24 @@ def read_ascii_data(dataf: Path) -> dict:
         while True:
             if not row:
                 break
-            while "#" * 8 not in row:  # start header section
-                row = f.readline()
             row = f.readline()  # skip ###### row
-            header = []
+            header_rows = []
             while "#" * 8 not in row:
-                header.append(row)
+                header_rows.append(row)
                 row = f.readline()
-            header = parse_header(header)
+                if not row:
+                    break
+            if len(header_rows) < 8:
+                raise ValueError("Unable to get header from file")
+            header = parse_header(header_rows)
 
             # parse body
-            body = []
+            body_rows = []
             row = f.readline()
             while row and "#" * 8 not in row:  # start header section
-                body.append(row)
+                body_rows.append(row)
                 row = f.readline()
-            body = parse_body(body)
+            body = parse_body(body_rows)
             header["data"] = body
             sections[header["variable"]] = header
     return sections
